@@ -7,21 +7,33 @@ Released under the GNU GPL version 3 or later
 
 '''
 
-import sys, os, struct, math, time, socket
+import sys, os, time, socket
 import fnmatch, errno, threading
 import serial, Queue, select
 import traceback
 import select
 
-# allow running without installing
-#sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..'))
-
-
 from MAVProxy.modules.lib import textconsole
 from MAVProxy.modules.lib import rline
 from MAVProxy.modules.lib import mp_module
 from MAVProxy.modules.lib import dumpstacks
 
+# adding all this allows pyinstaller to build a working windows executable
+# note that using --hidden-import does not work for these modules
+try:
+      from multiprocessing import freeze_support
+      from pymavlink import mavwp, mavutil
+      import wx, matplotlib, HTMLParser
+      try:
+            import readline
+      except ImportError:
+            import pyreadline as readline
+except Exception:
+      pass
+
+if __name__ == '__main__':
+      freeze_support()
+
 class MPStatus(object):
     '''hold status information about the mavproxy'''
     def __init__(self):
@@ -109,8 +121,8 @@ class MPState(object):
         from MAVProxy.modules.lib.mp_settings import MPSettings, MPSetting
         self.settings = MPSettings(
             [ MPSetting('link', int, 1, 'Primary Link', tab='Link', range=(0,4), increment=1),
-              MPSetting('streamrate', int, 4, 'Stream rate link1', range=(0,20), increment=1),
-              MPSetting('streamrate2', int, 4, 'Stream rate link2', range=(0,20), increment=1),
+              MPSetting('streamrate', int, 4, 'Stream rate link1', range=(-1,20), increment=1),
+              MPSetting('streamrate2', int, 4, 'Stream rate link2', range=(-1,20), increment=1),
               MPSetting('heartbeat', int, 1, 'Heartbeat rate', range=(0,5), increment=1),
               MPSetting('mavfwd', bool, True, 'Allow forwarded control'),
               MPSetting('mavfwd_rate', bool, False, 'Allow forwarded rate control'),
@@ -127,7 +139,13 @@ class MPState(object):
               MPSetting('basealt', int, 0, 'Base Altitude', range=(0,30000), increment=1, tab='Altitude'),
               MPSetting('wpalt', int, 100, 'Default WP Altitude', range=(0,10000), increment=1),
               MPSetting('rallyalt', int, 90, 'Default Rally Altitude', range=(0,10000), increment=1),
-              MPSetting('terrainalt', bool, False, 'Use terrain altitudes')]
+              MPSetting('terrainalt', str, 'Auto', 'Use terrain altitudes', choice=['Auto','True','False']),
+              MPSetting('wpupdates', bool, True, 'Show waypoint updates'),
+              MPSetting('rally_breakalt', int, 40, 'Default Rally Break Altitude', range=(0,10000), increment=1),
+              MPSetting('rally_flags', int, 0, 'Default Rally Flags`', range=(0,10000), increment=1),
+              MPSetting('baudrate', int, opts.baudrate, 'baudrate for new links', range=(0,10000000), increment=1),
+              MPSetting('rtscts', bool, opts.rtscts, 'enable flow control'),
+              MPSetting('select_timeout', float, 0.01, 'select timeout')]
             )
 
         self.completions = {
@@ -174,10 +192,6 @@ class MPState(object):
         return self.mav_master[self.settings.link-1]
 
 
-def get_usec():
-    '''time since 1970 in microseconds'''
-    return int(time.time() * 1.0e6)
-
 def get_mav_param(param, default=None):
     '''return a EEPROM parameter value'''
     return mpstate.mav_param.get(param, default)
@@ -216,18 +230,6 @@ def cmd_reset(args):
     print("Resetting master")
     mpstate.master().reset()
 
-def cmd_link(args):
-    for master in mpstate.mav_master:
-        linkdelay = (mpstate.status.highest_msec - master.highest_msec)*1.0e-3
-        if master.linkerror:
-            print("link %u down" % (master.linknum+1))
-        else:
-            print("link %u OK (%u packets, %.2fs delay, %u lost, %.1f%% loss)" % (master.linknum+1,
-                                                                                  mpstate.status.counters['MasterIn'][master.linknum],
-                                                                                  linkdelay,
-                                                                                  master.mav_loss,
-                                                                                  master.packet_loss()))
-
 def cmd_watch(args):
     '''watch a mavlink packet pattern'''
     if len(args) == 0:
@@ -304,7 +306,12 @@ def cmd_module(args):
             print("Module %s not loaded" % modname)
             return
         if unload_module(modname):
-            reload(pmodule)
+            import zipimport
+            try:
+                reload(pmodule)
+            except ImportError:
+                clear_zipimport_cache()
+                reload(pmodule)                
             if load_module(modname, quiet=True):
                 print("Reloaded module %s" % modname)
     elif args[0] == "unload":
@@ -347,13 +354,32 @@ def cmd_alias(args):
         print(usage)
         return
 
+
+def clear_zipimport_cache():
+    """Clear out cached entries from _zip_directory_cache.
+    See http://www.digi.com/wiki/developer/index.php/Error_messages"""
+    import sys, zipimport
+    syspath_backup = list(sys.path)
+    zipimport._zip_directory_cache.clear()
+ 
+    # load back items onto sys.path
+    sys.path = syspath_backup
+    # add this too: see https://mail.python.org/pipermail/python-list/2005-May/353229.html
+    sys.path_importer_cache.clear()
+
 # http://stackoverflow.com/questions/211100/pythons-import-doesnt-work-as-expected
 # has info on why this is necessary.
 
 def import_package(name):
     """Given a package name like 'foo.bar.quux', imports the package
     and returns the desired module."""
-    mod = __import__(name)
+    import zipimport
+    try:
+        mod = __import__(name)
+    except ImportError:
+        clear_zipimport_cache()
+        mod = __import__(name)
+        
     components = name.split('.')
     for comp in components[1:]:
         mod = getattr(mod, comp)
@@ -366,7 +392,6 @@ command_map = {
     'reset'   : (cmd_reset,    'reopen the connection to the MAVLink master'),
     'status'  : (cmd_status,   'show status'),
     'set'     : (cmd_set,      'mavproxy settings'),
-    'link'    : (cmd_link,     'show link status'),
     'watch'   : (cmd_watch,    'watch a MAVLink pattern'),
     'module'  : (cmd_module,   'module commands'),
     'alias'   : (cmd_alias,    'command aliases')
@@ -410,9 +435,7 @@ def process_stdin(line):
             print("%-15s : %s" % (cmd, help))
         return
     if cmd == 'exit' and mpstate.settings.requireexit:
-        reply = raw_input("Are you sure you want to exit? (y/N)")
-        if reply.lower() == 'y':
-            mpstate.status.exit = True
+        mpstate.status.exit = True
         return
 
     if not cmd in command_map:
@@ -434,239 +457,6 @@ def process_stdin(line):
             traceback.print_exc()
 
 
-def handle_msec_timestamp(m, master):
-    '''special handling for MAVLink packets with a time_boot_ms field'''
-
-    if m.get_type() == 'GLOBAL_POSITION_INT':
-        # this is fix time, not boot time
-        return
-
-    msec = m.time_boot_ms
-    if msec + 30000 < master.highest_msec:
-        say('Time has wrapped')
-        print('Time has wrapped', msec, master.highest_msec)
-        mpstate.status.highest_msec = msec
-        for mm in mpstate.mav_master:
-            mm.link_delayed = False
-            mm.highest_msec = msec
-        return
-
-    # we want to detect when a link is delayed
-    master.highest_msec = msec
-    if msec > mpstate.status.highest_msec:
-        mpstate.status.highest_msec = msec
-    if msec < mpstate.status.highest_msec and len(mpstate.mav_master) > 1:
-        master.link_delayed = True
-    else:
-        master.link_delayed = False
-
-def report_altitude(altitude):
-    '''possibly report a new altitude'''
-    master = mpstate.master()
-    if getattr(mpstate.console, 'ElevationMap', None) is not None and mpstate.settings.basealt != 0:
-        lat = master.field('GLOBAL_POSITION_INT', 'lat', 0)*1.0e-7
-        lon = master.field('GLOBAL_POSITION_INT', 'lon', 0)*1.0e-7
-        alt1 = mpstate.console.ElevationMap.GetElevation(lat, lon)
-        if alt1 is not None:
-            alt2 = mpstate.settings.basealt
-            altitude += alt2 - alt1
-    mpstate.status.altitude = altitude
-    if (int(mpstate.settings.altreadout) > 0 and
-        math.fabs(mpstate.status.altitude - mpstate.status.last_altitude_announce) >= int(mpstate.settings.altreadout)):
-        mpstate.status.last_altitude_announce = mpstate.status.altitude
-        rounded_alt = int(mpstate.settings.altreadout) * ((mpstate.settings.altreadout/2 + int(mpstate.status.altitude)) / int(mpstate.settings.altreadout))
-        say("height %u" % rounded_alt, priority='notification')
-
-
-def master_send_callback(m, master):
-    '''called on sending a message'''
-    mtype = m.get_type()
-
-    if mtype != 'BAD_DATA' and mpstate.logqueue:
-        usec = get_usec()
-        usec = (usec & ~3) | 3 # linknum 3
-        mpstate.logqueue.put(str(struct.pack('>Q', usec) + m.get_msgbuf()))
-
-
-def master_callback(m, master):
-    '''process mavlink message m on master, sending any messages to recipients'''
-
-    if getattr(m, '_timestamp', None) is None:
-        master.post_message(m)
-    mpstate.status.counters['MasterIn'][master.linknum] += 1
-
-    if getattr(m, 'time_boot_ms', None) is not None:
-        # update link_delayed attribute
-        handle_msec_timestamp(m, master)
-
-    mtype = m.get_type()
-
-    # and log them
-    if mtype not in ['BAD_DATA','LOG_DATA'] and mpstate.logqueue:
-        # put link number in bottom 2 bits, so we can analyse packet
-        # delay in saved logs
-        usec = get_usec()
-        usec = (usec & ~3) | master.linknum
-        mpstate.logqueue.put(str(struct.pack('>Q', usec) + m.get_msgbuf()))
-
-    if mtype in [ 'HEARTBEAT', 'GPS_RAW_INT', 'GPS_RAW', 'GLOBAL_POSITION_INT', 'SYS_STATUS' ]:
-        if master.linkerror:
-            master.linkerror = False
-            say("link %u OK" % (master.linknum+1))
-        mpstate.status.last_message = time.time()
-        master.last_message = mpstate.status.last_message
-
-    if master.link_delayed:
-        # don't process delayed packets that cause double reporting
-        if mtype in [ 'MISSION_CURRENT', 'SYS_STATUS', 'VFR_HUD',
-                      'GPS_RAW_INT', 'SCALED_PRESSURE', 'GLOBAL_POSITION_INT',
-                      'NAV_CONTROLLER_OUTPUT' ]:
-            return
-
-    if mtype == 'HEARTBEAT' and m.get_srcSystem() != 255:
-        if (mpstate.status.target_system != m.get_srcSystem() or
-            mpstate.status.target_component != m.get_srcComponent()):
-            mpstate.status.target_system = m.get_srcSystem()
-            mpstate.status.target_component = m.get_srcComponent()
-            say("online system %u component %u" % (mpstate.status.target_system, mpstate.status.target_component),'message')
-
-        if mpstate.status.heartbeat_error:
-            mpstate.status.heartbeat_error = False
-            say("heartbeat OK")
-        if master.linkerror:
-            master.linkerror = False
-            say("link %u OK" % (master.linknum+1))
-
-        mpstate.status.last_heartbeat = time.time()
-        master.last_heartbeat = mpstate.status.last_heartbeat
-
-        armed = mpstate.master().motors_armed()
-        if armed != mpstate.status.armed:
-            mpstate.status.armed = armed
-            if armed:
-                say("ARMED")
-            else:
-                say("DISARMED")
-
-        if master.flightmode != mpstate.status.flightmode and time.time() > mpstate.status.last_mode_announce + 2:
-            mpstate.status.flightmode = master.flightmode
-            mpstate.status.last_mode_announce = time.time()
-            mpstate.rl.set_prompt(mpstate.status.flightmode + "> ")
-            say("Mode " + mpstate.status.flightmode)
-
-        if m.type in [mavutil.mavlink.MAV_TYPE_FIXED_WING]:
-            mpstate.vehicle_type = 'plane'
-            mpstate.vehicle_name = 'ArduPlane'
-        elif m.type in [mavutil.mavlink.MAV_TYPE_GROUND_ROVER,
-                        mavutil.mavlink.MAV_TYPE_SURFACE_BOAT,
-                        mavutil.mavlink.MAV_TYPE_SUBMARINE]:
-            mpstate.vehicle_type = 'rover'
-            mpstate.vehicle_name = 'APMrover2'
-        elif m.type in [mavutil.mavlink.MAV_TYPE_QUADROTOR,
-                        mavutil.mavlink.MAV_TYPE_COAXIAL,
-                        mavutil.mavlink.MAV_TYPE_HEXAROTOR,
-                        mavutil.mavlink.MAV_TYPE_OCTOROTOR,
-                        mavutil.mavlink.MAV_TYPE_TRICOPTER,
-                        mavutil.mavlink.MAV_TYPE_HELICOPTER]:
-            mpstate.vehicle_type = 'copter'
-            mpstate.vehicle_name = 'ArduCopter'
-        elif m.type in [mavutil.mavlink.MAV_TYPE_ANTENNA_TRACKER]:
-            mpstate.vehicle_type = 'antenna'
-            mpstate.vehicle_name = 'AntennaTracker'
-        
-    elif mtype == 'STATUSTEXT':
-        if m.text != mpstate.status.last_apm_msg or time.time() > mpstate.status.last_apm_msg_time+2:
-            mpstate.console.writeln("APM: %s" % m.text, bg='red')
-            mpstate.status.last_apm_msg = m.text
-            mpstate.status.last_apm_msg_time = time.time()
-
-    elif mtype == "VFR_HUD":
-        have_gps_fix = False
-        if 'GPS_RAW' in mpstate.status.msgs and mpstate.status.msgs['GPS_RAW'].fix_type == 2:
-            have_gps_fix = True
-        if 'GPS_RAW_INT' in mpstate.status.msgs and mpstate.status.msgs['GPS_RAW_INT'].fix_type == 3:
-            have_gps_fix = True
-        if have_gps_fix and not mpstate.status.have_gps_lock and m.alt != 0:
-                say("GPS lock at %u meters" % m.alt, priority='notification')
-                mpstate.status.have_gps_lock = True
-
-    elif mtype == "GPS_RAW":
-        if mpstate.status.have_gps_lock:
-            if m.fix_type != 2 and not mpstate.status.lost_gps_lock and (time.time() - mpstate.status.last_gps_lock) > 3:
-                say("GPS fix lost")
-                mpstate.status.lost_gps_lock = True
-            if m.fix_type == 2 and mpstate.status.lost_gps_lock:
-                say("GPS OK")
-                mpstate.status.lost_gps_lock = False
-            if m.fix_type == 2:
-                mpstate.status.last_gps_lock = time.time()
-
-    elif mtype == "GPS_RAW_INT":
-        if mpstate.status.have_gps_lock:
-            if m.fix_type < 3 and not mpstate.status.lost_gps_lock and (time.time() - mpstate.status.last_gps_lock) > 3:
-                say("GPS fix lost")
-                mpstate.status.lost_gps_lock = True
-            if m.fix_type >= 3 and mpstate.status.lost_gps_lock:
-                say("GPS OK")
-                mpstate.status.lost_gps_lock = False
-            if m.fix_type >= 3:
-                mpstate.status.last_gps_lock = time.time()
-
-    elif mtype == "NAV_CONTROLLER_OUTPUT" and mpstate.status.flightmode == "AUTO" and mpstate.settings.distreadout:
-        rounded_dist = int(m.wp_dist/mpstate.settings.distreadout)*mpstate.settings.distreadout
-        if math.fabs(rounded_dist - mpstate.status.last_distance_announce) >= mpstate.settings.distreadout:
-            if rounded_dist != 0:
-                say("%u" % rounded_dist, priority="progress")
-            mpstate.status.last_distance_announce = rounded_dist
-    
-    elif mtype == "GLOBAL_POSITION_INT":
-        report_altitude(m.relative_alt*0.001)
-
-    elif mtype == "COMPASSMOT_STATUS":
-        print(m)
-
-    elif mtype == "BAD_DATA":
-        if mpstate.settings.shownoise and mavutil.all_printable(m.data):
-            mpstate.console.write(str(m.data), bg='red')
-    elif mtype in [ "COMMAND_ACK", "MISSION_ACK" ]:
-        mpstate.console.writeln("Got MAVLink msg: %s" % m)
-    else:
-        #mpstate.console.writeln("Got MAVLink msg: %s" % m)
-        pass
-
-    if mpstate.status.watch is not None:
-        if fnmatch.fnmatch(m.get_type().upper(), mpstate.status.watch.upper()):
-            mpstate.console.writeln(m)
-
-    # keep the last message of each type around
-    mpstate.status.msgs[m.get_type()] = m
-    if not m.get_type() in mpstate.status.msg_count:
-        mpstate.status.msg_count[m.get_type()] = 0
-    mpstate.status.msg_count[m.get_type()] += 1
-
-    # don't pass along bad data
-    if mtype != "BAD_DATA":
-        # pass messages along to listeners, except for REQUEST_DATA_STREAM, which
-        # would lead a conflict in stream rate setting between mavproxy and the other
-        # GCS
-        if mpstate.settings.mavfwd_rate or mtype != 'REQUEST_DATA_STREAM':
-            for r in mpstate.mav_outputs:
-                r.write(m.get_msgbuf())
-
-        # pass to modules
-        for (mod,pm) in mpstate.modules:
-            if not hasattr(mod, 'mavlink_packet'):
-                continue
-            try:
-                mod.mavlink_packet(m)
-            except Exception as msg:
-                if mpstate.settings.moddebug == 1:
-                    print(msg)
-                elif mpstate.settings.moddebug > 1:
-                    exc_type, exc_value, exc_traceback = sys.exc_info()
-                    traceback.print_exception(exc_type, exc_value, exc_traceback,
-                                              limit=2, file=sys.stdout)
-
 def process_master(m):
     '''process packets from the MAVLink master'''
     try:
@@ -817,7 +607,7 @@ def check_link_status():
         say("no link")
         mpstate.status.heartbeat_error = True
     for master in mpstate.mav_master:
-        if not master.linkerror and tnow > master.last_message + 5:
+        if not master.linkerror and (tnow > master.last_message + 5 or master.portdead):
             say("link %u down" % (master.linknum+1))
             master.linkerror = True
 
@@ -867,7 +657,8 @@ def main_loop():
     if not mpstate.status.setup_mode and not opts.nowait:
         for master in mpstate.mav_master:
             send_heartbeat(master)
-            master.wait_heartbeat()
+            if master.linknum == 0:
+                master.wait_heartbeat()
         set_stream_rates()
 
     while True:
@@ -889,7 +680,7 @@ def main_loop():
 
         rin = []
         for master in mpstate.mav_master:
-            if master.fd is not None:
+            if master.fd is not None and not master.portdead:
                 rin.append(master.fd)
         for m in mpstate.mav_outputs:
             rin.append(m.fd)
@@ -900,7 +691,7 @@ def main_loop():
         for fd in mpstate.select_extra:
             rin.append(fd)
         try:
-            (rin, win, xin) = select.select(rin, [], [], 0.01)
+            (rin, win, xin) = select.select(rin, [], [], mpstate.settings.select_timeout)
         except select.error:
             continue
 
@@ -998,7 +789,7 @@ if __name__ == '__main__':
     parser.add_option("--speech", dest="speech", help="use text to speach",
                       action='store_true', default=False)
     parser.add_option("--aircraft", dest="aircraft", help="aircraft name", default=None)
-    parser.add_option("--cmd", dest="cmd", help="initial commands", default=None)
+    parser.add_option("--cmd", dest="cmd", help="initial commands", default=None, action='append')
     parser.add_option("--console", action='store_true', help="use GUI console")
     parser.add_option("--map", action='store_true', help="load map module")
     parser.add_option(
@@ -1009,7 +800,7 @@ if __name__ == '__main__':
     parser.add_option("--mav09", action='store_true', default=False, help="Use MAVLink protocol 0.9")
     parser.add_option("--auto-protocol", action='store_true', default=False, help="Auto detect MAVLink protocol version")
     parser.add_option("--nowait", action='store_true', default=False, help="don't wait for HEARTBEAT on startup")
-    parser.add_option("--continue", dest='continue_mode', action='store_true', default=False, help="continue logs")
+    parser.add_option("-c", "--continue", dest='continue_mode', action='store_true', default=False, help="continue logs")
     parser.add_option("--dialect",  default="ardupilotmega", help="MAVLink dialect")
     parser.add_option("--rtscts",  action='store_true', help="enable hardware RTS/CTS flow control")
     parser.add_option("--mission", dest="mission", help="mission name", default=None)
@@ -1037,7 +828,7 @@ if __name__ == '__main__':
         load_module('speech')
 
     if not opts.master:
-        serial_list = mavutil.auto_detect_serial(preferred_list=['*FTDI*',"*Arduino_Mega_2560*", "*3D_Robotics*", "*USB_to_UART*"])
+        serial_list = mavutil.auto_detect_serial(preferred_list=['*FTDI*',"*Arduino_Mega_2560*", "*3D_Robotics*", "*USB_to_UART*", '*PX4*', '*FMU*'])
         if len(serial_list) == 1:
             opts.master = [serial_list[0].device]
         else:
@@ -1060,39 +851,19 @@ Auto-detected serial ports are:
 
     mpstate.mav_master = []
 
+    load_module('link', quiet=True)
+
     # open master link
     for mdev in opts.master:
-        if ',' in mdev and not os.path.exists(mdev):
-            port, baud = mdev.split(',')
-        else:
-            port, baud = mdev, opts.baudrate
-
-        m = mavutil.mavlink_connection(port, autoreconnect=True, baud=int(baud))
-        m.mav.set_callback(master_callback, m)
-        if hasattr(m.mav, 'set_send_callback'):
-            m.mav.set_send_callback(master_send_callback, m)
-        if opts.rtscts:
-            m.set_rtscts(True)
-        m.linknum = len(mpstate.mav_master)
-        m.linkerror = False
-        m.link_delayed = False
-        m.last_heartbeat = 0
-        m.last_message = 0
-        m.highest_msec = 0
-        mpstate.mav_master.append(m)
-        mpstate.status.counters['MasterIn'].append(0)
+        if not mpstate.module('link').link_add(mdev):
+            sys.exit(1)
 
     # log all packets from the master, for later replay
     open_logs()
 
-    # open any mavlink UDP ports
-    for p in opts.output:
-        if ',' in p and not os.path.exists(p):
-            port, baud = p.split(',')            
-        else:
-            port, baud = p, opts.baudrate
-
-        mpstate.mav_outputs.append(mavutil.mavlink_connection(port, baud=int(baud), input=False))
+    # open any mavlink output ports
+    for port in opts.output:
+        mpstate.mav_outputs.append(mavutil.mavlink_connection(port, baud=int(opts.baudrate), input=False))
 
     if opts.sitl:
         mpstate.sitl_output = mavutil.mavudp(opts.sitl, input=False)
@@ -1110,10 +881,22 @@ Auto-detected serial ports are:
     if opts.setup:
         mpstate.rl.set_prompt("")
 
+    if not opts.setup:
+        # some core functionality is in modules
+        standard_modules = ['log', 'wp', 'rally','fence','param','relay',
+                            'tuneopt','arm','mode','calibration','rc','auxopt','misc','cmdlong',
+                            'battery','terrain','output']
+        for m in standard_modules:
+            load_module(m, quiet=True)
+
     if 'HOME' in os.environ and not opts.setup:
         start_script = os.path.join(os.environ['HOME'], ".mavinit.scr")
         if os.path.exists(start_script):
             run_script(start_script)
+    if 'LOCALAPPDATA' in os.environ and not opts.setup:
+        start_script = os.path.join(os.environ['LOCALAPPDATA'], "MAVProxy", "mavinit.scr")
+        if os.path.exists(start_script):
+            run_script(start_script)
 
     if opts.aircraft is not None:
         start_script = os.path.join(opts.aircraft, "mavinit.scr")
@@ -1122,14 +905,6 @@ Auto-detected serial ports are:
         else:
             print("no script %s" % start_script)
 
-    if not opts.setup:
-        # some core functionality is in modules
-        standard_modules = ['log', 'wp', 'rally','fence','param','relay',
-                            'tuneopt','arm','mode','calibration','rc','auxopt','misc','cmdlong',
-                            'battery','terrain']
-        for m in standard_modules:
-            load_module(m, quiet=True)
-
     if opts.console:
         process_stdin('module load console')
 
@@ -1142,9 +917,10 @@ Auto-detected serial ports are:
             process_stdin('module load %s' % mod)
 
     if opts.cmd is not None:
-        cmds = opts.cmd.split(';')
-        for c in cmds:
-            process_stdin(c)
+        for cstr in opts.cmd:
+            cmds = cstr.split(';')
+            for c in cmds:
+                process_stdin(c)
 
     # run main loop as a thread
     mpstate.status.thread = threading.Thread(target=main_loop)
