@@ -2,7 +2,7 @@
 readline handling for mavproxy
 '''
 
-import sys, glob
+import sys, glob, os
 
 rline_mpstate = None
 
@@ -12,7 +12,6 @@ class rline(object):
         import threading
         global rline_mpstate
         self.prompt = prompt
-        self.line = None
         rline_mpstate = mpstate
         # other modules can add their own completion functions
         mpstate.completion_functions = {
@@ -23,7 +22,7 @@ class rline(object):
             '(COMMAND)' : complete_command,
             '(ALIAS)' : complete_alias
             }
-        
+
     def set_prompt(self, prompt):
         if prompt != self.prompt:
             self.prompt = prompt
@@ -42,7 +41,14 @@ def complete_command(text):
 
 def complete_filename(text):
     '''complete a filename'''
-    return glob.glob(text+'*')
+
+    #ensure directories have trailing slashes:
+    list = glob.glob(text+'*')
+    for idx, val in enumerate(list):
+        if os.path.isdir(val):
+            list[idx] = (val + os.path.sep)
+
+    return list
 
 def complete_parameter(text):
     '''complete a parameter'''
@@ -67,7 +73,7 @@ def rule_expand(component, text):
         return component[1:-1].split('|')
     if component in rline_mpstate.completion_functions:
         return rline_mpstate.completion_functions[component](text)
-    return [component]        
+    return [component]
 
 def rule_match(component, cmd):
     '''see if one rule component matches'''
@@ -76,7 +82,7 @@ def rule_match(component, cmd):
     expanded = rule_expand(component, cmd)
     if cmd in expanded:
         return True
-    return False      
+    return False
 
 def complete_rule(rule, cmd):
     '''complete using one rule'''
@@ -91,7 +97,7 @@ def complete_rule(rule, cmd):
     # expand the next rule component
     expanded = rule_expand(rule_components[len(cmd)-1], cmd[-1])
     return expanded
-    
+
 
 def complete_rules(rules, cmd):
     '''complete using a list of completion rules'''
@@ -101,7 +107,7 @@ def complete_rules(rules, cmd):
     for r in rules:
         ret += complete_rule(r, cmd)
     return ret
-   
+
 
 last_clist = None
 
@@ -111,7 +117,7 @@ def complete(text, state):
     global rline_mpstate
     if state != 0 and last_clist is not None:
         return last_clist[state]
-        
+
     # split the command so far
     cmd = readline.get_line_buffer().split(' ')
 
@@ -131,15 +137,43 @@ def complete(text, state):
     ret.append(None)
     last_clist = ret
     return last_clist[state]
-    
-    
+
+
 
 # some python distributions don't have readline, so handle that case
 # with a try/except
 try:
-    import readline
+    try:
+        import readline
+    except ImportError:
+        import pyreadline as readline
     readline.set_completer_delims(' \t\n;')
     readline.parse_and_bind("tab: complete")
     readline.set_completer(complete)
 except Exception:
     pass
+
+
+if __name__ == "__main__":
+    from mp_settings import MPSettings, MPSetting
+    
+    class mystate(object):
+        def __init__(self):
+            self.settings = MPSettings(
+            [ MPSetting('foo', int, 1, 'foo int', tab='Link', range=(0,4), increment=1),
+              MPSetting('bar', float, 4, 'bar float', range=(-1,20), increment=1)])
+            self.completions = {
+                "script" : ["(FILENAME)"],
+                "set"    : ["(SETTING)"]
+                }
+            self.command_map = {
+                'script'  : (None,   'run a script of MAVProxy commands'),
+                'set'     : (None,   'mavproxy settings'),
+                }
+            self.aliases = {}
+
+    state = mystate()
+    rl = rline("test> ", state)
+    while True:
+        line = raw_input(rl.prompt)
+        print("Got: %s" % line)
