@@ -41,8 +41,9 @@ class MapModule(mp_module.MPModule):
               ('showgps2pos', int, 1),
               ('showsimpos', int, 0),
               ('showahrs2pos', int, 0),
-              ('brightness', float, 1)])
-        service='YahooSat'
+              ('brightness', float, 1),
+              ('rallycircle', bool, False)])
+        service='OviHybrid'
         if 'MAP_SERVICE' in os.environ:
             service = os.environ['MAP_SERVICE']
         import platform
@@ -58,12 +59,27 @@ class MapModule(mp_module.MPModule):
         self.add_menu(MPMenuItem('Fly To', 'Fly To', '# guided ',
                                  handler=MPMenuCallTextDialog(title='Altitude (m)', default=100)))
         self.add_menu(MPMenuItem('Terrain Check', 'Terrain Check', '# terrain check'))
+        self.add_menu(MPMenuItem('Show Position', 'Show Position', 'showPosition'))
 
     def add_menu(self, menu):
         '''add to the default popup menu'''
         self.default_popup.add(menu)
         self.mpstate.map.add_object(mp_slipmap.SlipDefaultPopup(self.default_popup, combine=True))
 
+    def show_position(self):
+        '''show map position click information'''
+        pos = self.click_position
+        dms = (mp_util.degrees_to_dms(pos[0]), mp_util.degrees_to_dms(pos[1]))
+        msg =  "Coordinates in WGS84\n"
+        msg += "Decimal: %.6f %.6f\n" % (pos[0], pos[1])
+        msg += "DMS:     %s %s\n" % (dms[0], dms[1])
+        msg += "Grid:    %s\n" % mp_util.latlon_to_grid(pos)
+        if self.logdir:
+            logf = open(os.path.join(self.logdir, "positions.txt"), "a")
+            logf.write("Position: %.6f %.6f at %s\n" % (pos[0], pos[1], time.ctime()))
+            logf.close()
+        MPMenuChildMessageDialog('Position', msg, font_size=32)
+
     def cmd_map(self, args):
         '''map commands'''
         if args[0] == "icon":
@@ -101,6 +117,16 @@ class MapModule(mp_module.MPModule):
                 self.mpstate.map.add_object(mp_slipmap.SlipPolygon('mission %u' % i, p,
                                                                    layer='Mission', linewidth=2, colour=(255,255,255),
                                                                    popup_menu=popup))
+        labeled_wps = {}
+        for i in range(len(self.mission_list)):
+            next_list = self.mission_list[i]
+            for j in range(len(next_list)):
+                #label already printed for this wp?
+                if (next_list[j] not in labeled_wps):
+                    self.mpstate.map.add_object(mp_slipmap.SlipLabel(
+                        'miss_cmd %u/%u' % (i,j), polygons[i][j], str(next_list[j]), 'Mission', colour=(0,255,255)))  
+                    labeled_wps[next_list[j]] = (i,j)
+                    
 
     def display_fence(self):
         '''display the fence'''
@@ -215,6 +241,8 @@ class MapModule(mp_module.MPModule):
             self.remove_fencepoint(obj.selected[0].objkey, obj.selected[0].extra_info)
         elif menuitem.returnkey == 'popupFenceMove':
             self.move_fencepoint(obj.selected[0].objkey, obj.selected[0].extra_info)
+        elif menuitem.returnkey == 'showPosition':
+            self.show_position()
 
     def map_callback(self, obj):
         '''called when an event happens on the slipmap'''
@@ -259,6 +287,10 @@ class MapModule(mp_module.MPModule):
                 self.click_position = obj.latlon
                 self.click_time = time.time()
                 self.drawing_update()
+
+            if self.module('misseditor') is not None:
+                self.module('misseditor').update_map_click_position(self.click_position)
+
         if obj.event.m_rightDown:
             if self.draw_callback is not None:
                 self.drawing_end()
@@ -370,9 +402,13 @@ class MapModule(mp_module.MPModule):
     
             
         # if the waypoints have changed, redisplay
-        if self.wp_change_time != self.module('wp').wploader.last_change:
-            self.wp_change_time = self.module('wp').wploader.last_change
+        last_wp_change = self.module('wp').wploader.last_change
+        if self.wp_change_time != last_wp_change and abs(time.time() - last_wp_change) > 1:
+            self.wp_change_time = last_wp_change
             self.display_waypoints()
+
+            #this may have affected the landing lines from the rally points:
+            self.rally_change_time = time.time()
     
         # if the fence has changed, redisplay
         if self.fence_change_time != self.module('fence').fenceloader.last_change:
@@ -391,7 +427,41 @@ class MapModule(mp_module.MPModule):
                 self.mpstate.map.add_object(mp_slipmap.SlipIcon('Rally %u' % (i+1), (rp.lat*1.0e-7, rp.lng*1.0e-7), icon,
                                                                 layer='RallyPoints', rotation=0, follow=False,
                                                                 popup_menu=popup))
-    
+
+                loiter_rad = self.get_mav_param('WP_LOITER_RAD')
+
+                if self.map_settings.rallycircle:
+                    self.mpstate.map.add_object(mp_slipmap.SlipCircle('Rally Circ %u' % (i+1), 'RallyPoints', (rp.lat*1.0e-7, rp.lng*1.0e-7), abs(loiter_rad), (255,255,0), 2))
+
+                #draw a line between rally point and nearest landing point
+                nearest_land_wp = None
+                nearest_distance = 10000000.0
+                for j in range(self.module('wp').wploader.count()):
+                    w = self.module('wp').wploader.wp(j)
+                    if (w.command == 21): #if landing waypoint
+                        #get distance between rally point and this waypoint
+                        dis = mp_util.gps_distance(w.x, w.y, rp.lat*1.0e-7, rp.lng*1.0e-7)
+                        if (dis < nearest_distance):
+                            nearest_land_wp = w
+                            nearest_distance = dis
+
+                if nearest_land_wp != None:
+                    points = []
+                    #tangential approach?
+                    if self.get_mav_param('LAND_BREAK_PATH') == 0:
+                        theta = math.degrees(math.atan(loiter_rad / nearest_distance))
+                        tan_dis = math.sqrt(nearest_distance * nearest_distance - (loiter_rad * loiter_rad))
+
+                        ral_bearing = mp_util.gps_bearing(nearest_land_wp.x, nearest_land_wp.y,rp.lat*1.0e-7, rp.lng*1.0e-7)
+
+                        points.append(mp_util.gps_newpos(nearest_land_wp.x,nearest_land_wp.y, ral_bearing + theta, tan_dis))
+
+                    else: #not tangential approach
+                        points.append((rp.lat*1.0e-7, rp.lng*1.0e-7))
+
+                    points.append((nearest_land_wp.x, nearest_land_wp.y))
+                    self.mpstate.map.add_object(mp_slipmap.SlipPolygon('Rally Land %u' % (i+1), points, 'RallyPoints', (255,255,0), 2))
+
         # check for any events from the map
         self.mpstate.map.check_events()
     
